﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace Mediachase.Commerce.Website.Controls
{
    /// <summary>
    /// Loads the control template using the theme specified.
    /// </summary>
    public class ThemedControl : PlaceHolder
    {
        private string _ThemePath = "";
        private System.Web.UI.Control ctrl = null;


        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Load"/> event.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            CreateChildControlsTree();
        }

        /// <summary>
        /// Handles the Load event of the Page control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, System.EventArgs e)
        {
            // Make sure controls are created before the viewstate is initialized
            // TPB: fix, this control does not need to keep viewstate so init it in page_load
            // CreateChildControlsTree();
        }

        /// <summary>
        /// Gets or sets the theme path.
        /// </summary>
        /// <value>The theme path.</value>
        public string ThemePath
        {
            get
            {
                return _ThemePath;
            }
            set
            {
                _ThemePath = value;
            }
        }

        /// <summary>
        /// Gets or sets the site theme.
        /// </summary>
        /// <value>The site theme.</value>
        public string SiteTheme {get;set;}


        string _DefaultSiteTheme = "Default";
        /// <summary>
        /// Gets or sets the default site theme.
        /// </summary>
        /// <value>The site theme.</value>
        public string DefaultSiteTheme
        {
            get
            {
                return _DefaultSiteTheme;
            }
            set
            {
                _DefaultSiteTheme = value;
            }
        }

        /// <summary>
        /// Gets the control.
        /// </summary>
        /// <value>The control.</value>
        public System.Web.UI.Control Control
        {
            get
            {
                return ctrl;
            }
        }

        /// <summary>
        /// Creates child controls before viewstate is initialized.
        /// </summary>
        private void CreateChildControlsTree()
        {
            string id = "Theme" + this.ID;
            ctrl = this.FindControl(id);
            
            if (ctrl == null)
            {
                if (System.IO.File.Exists(this.Page.MapPath("~/Templates/" + this.SiteTheme + "/" + _ThemePath)))
                    ctrl = this.Page.LoadControl("~/Templates/" + this.SiteTheme + "/" + _ThemePath);
                else
                    ctrl = this.Page.LoadControl("~/Templates/" + DefaultSiteTheme + "/" + _ThemePath);

                ctrl.ID = id;
                this.Controls.Add(ctrl);
            }

            // we should not call databind since we just adding controls here
            // allow time to restore the viewstate if needed
            // ctrl.DataBind();
        }

    }
}
